@@ -24,19 +24,26 @@
 
 #include "mt-common.h"
 
+#define WID(n) (glade_xml_get_widget (dd->xml, (n)))
+
 typedef struct _DwellData DwellData;
 struct _DwellData {
     GConfClient *client;
     DBusGProxy  *proxy;
     GladeXML    *xml;
     GtkWidget   *box;
+    GtkWidget   *ct_box;
+    GtkWidget   *enable;
     GtkWidget   *button;
     GdkPixbuf   *click[4];
-
-    gint button_width;
-    gint button_height;
-    gint cct;
-    gint active;
+    gint         button_width;
+    gint         button_height;
+    gint         cct;
+    gboolean     active;
+    GTimer      *timer;
+    guint        tid;
+    gdouble      delay;
+    gdouble      elapsed;
 };
 
 static const gchar *img_widgets[] = {
@@ -53,17 +60,15 @@ static const gchar *img_widgets_v[] = {
     "single_click_img_v"
 };
 
-static void update_sensitivity (DwellData *dd);
-
 static void preferences_dialog (BonoboUIComponent *component,
-				gpointer data,
-				const char *cname);
+				gpointer           data,
+				const char        *cname);
 static void help_dialog        (BonoboUIComponent *component,
-				gpointer data,
-				const char *cname);
+				gpointer           data,
+				const char        *cname);
 static void about_dialog       (BonoboUIComponent *component,
-				gpointer data,
-				const char *cname);
+				gpointer           data,
+				const char        *cname);
 
 static const BonoboUIVerb menu_verb[] = {
     BONOBO_UI_UNSAFE_VERB ("PropertiesVerb", preferences_dialog),
@@ -72,6 +77,18 @@ static const BonoboUIVerb menu_verb[] = {
     BONOBO_UI_VERB_END
 };
 
+static void
+update_sensitivity (DwellData *dd)
+{
+    gboolean dwell, sensitive;
+    gint mode;
+
+    dwell = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dd->enable));
+    mode = gconf_client_get_int (dd->client, OPT_MODE, NULL);
+    sensitive = dd->active && dwell && mode == DWELL_MODE_CTW;
+    gtk_widget_set_sensitive (dd->ct_box, sensitive);
+}
+
 static gboolean
 do_not_eat (GtkWidget *widget, GdkEventButton *bev, gpointer user)
 {
@@ -81,10 +98,108 @@ do_not_eat (GtkWidget *widget, GdkEventButton *bev, gpointer user)
     return FALSE;
 }
 
+static void
+enable_dwell_changed (GtkToggleButton *button, gpointer data)
+{
+    DwellData *dd = data;
+    gboolean dwell;
+
+    /* disable click-type window if it's active */
+    gconf_client_set_bool (dd->client, OPT_CTW, FALSE, NULL);
+
+    dwell = gtk_toggle_button_get_active (button);
+    gconf_client_set_bool (dd->client, OPT_DWELL, dwell, NULL);
+
+    gtk_button_set_label (GTK_BUTTON (button), dwell ? _("On") : _("Off"));
+}
+
+static gboolean
+activation_timeout (gpointer data)
+{
+    DwellData *dd = data;
+    gboolean stop;
+
+    if ((dd->elapsed = g_timer_elapsed (dd->timer, NULL)) < dd->delay)
+	stop = TRUE;
+    else {
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dd->enable), TRUE);
+	dd->tid = 0;
+	dd->elapsed = 0.0;
+	stop = FALSE;
+    }
+    gtk_widget_queue_draw (dd->enable);
+
+    return stop;
+}
+
+static gboolean
+enable_dwell_crossing (GtkWidget        *widget,
+		       GdkEventCrossing *event,
+		       gpointer          data)
+{
+    DwellData *dd = data;
+    GError *error = NULL;
+
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dd->enable)))
+	return FALSE;
+
+    if (event->type == GDK_ENTER_NOTIFY) {
+	if (!dd->tid) {
+	    dd->delay = gconf_client_get_float (dd->client, OPT_DWELL_T, &error);
+	    if (error) {
+		g_error_free (error);
+		dd->delay = 1.2;
+	    }
+	    g_timer_start (dd->timer);
+	    dd->tid = g_timeout_add (100, activation_timeout, dd);
+	}
+    }
+    else {
+	if (dd->tid) {
+	    g_source_remove (dd->tid);
+	    dd->tid = 0;
+	    dd->elapsed = 0.0;
+	}
+    }
+    return FALSE;
+}
+
+static gboolean
+enable_dwell_exposed (GtkWidget      *widget,
+		      GdkEventExpose *event,
+		      gpointer        data)
+{
+    DwellData *dd = data;
+    cairo_t *cr;
+    GdkColor c;
+    gdouble x, y, w, h;
+    gint fwidth, fpad;
+
+    c = widget->style->bg[GTK_STATE_SELECTED];
+    gtk_widget_style_get (widget,
+			  "focus-line-width", &fwidth,
+			  "focus-padding", &fpad,
+			  NULL); 
+    x = widget->allocation.x + fwidth + fpad;
+    y = widget->allocation.y + fwidth + fpad;
+    w = widget->allocation.width - (fwidth + fpad) * 2;
+    h = widget->allocation.height - (fwidth + fpad) * 2;
+
+    cr = gdk_cairo_create (widget->window);
+    cairo_set_source_rgba (cr,
+			   c.red / 65535., c.green / 65535., c.blue / 65535.,
+			   0.5); 
+    cairo_rectangle (cr, x, y, w, (h * dd->elapsed) / dd->delay);
+    cairo_fill (cr);
+    cairo_destroy (cr);
+
+    return FALSE;
+}
+
 static void
 button_cb (GtkToggleButton *button, gpointer data)
 {
-    DwellData *dd = (DwellData *) data;
+    DwellData *dd = data;
 
     if (gtk_toggle_button_get_active (button)) {
 	GSList *group;
@@ -98,56 +213,57 @@ button_cb (GtkToggleButton *button, gpointer data)
 }
 
 static void
-box_size_allocate (GtkWidget *widget, GtkAllocation *alloc, gpointer data)
+button_size_allocate (GtkWidget     *widget,
+		      GtkAllocation *alloc,
+		      gpointer       data)
 {
-    DwellData *dd = (DwellData *) data;
+    DwellData *dd = data;
     GtkWidget *w;
     GdkPixbuf *tmp;
     const gchar *name;
     gint i;
 
-    if (dd->button_width == alloc->width && dd->button_height == alloc->height)
+    if (dd->button_width == alloc->width &&
+	dd->button_height == alloc->height)
 	return;
 
     name = glade_get_widget_name (dd->box);
-
-    if (g_str_equal (name, "box_vert"))
+    if (g_str_equal (name, "box_vert")) {
 	/* vertical */
 	for (i = 0; i < N_CLICK_TYPES; i++) {
-	    w = glade_xml_get_widget (dd->xml, img_widgets_v[i]);
-
+	    w = WID (img_widgets_v[i]);
 	    if (alloc->width < 32) {
 		tmp = gdk_pixbuf_scale_simple (dd->click[i],
 					       alloc->width - 7,
 					       alloc->width - 7,
 					       GDK_INTERP_HYPER);
 		if (tmp) {
-		    gtk_image_set_from_pixbuf (GTK_IMAGE(w), tmp);
+		    gtk_image_set_from_pixbuf (GTK_IMAGE (w), tmp);
 		    g_object_unref (tmp);
 		}
 	    }
 	    else
-		gtk_image_set_from_pixbuf (GTK_IMAGE(w), dd->click[i]);
+		gtk_image_set_from_pixbuf (GTK_IMAGE (w), dd->click[i]);
 	}
-    else
+    }
+    else {
 	/* horizontal */
 	for (i = 0; i < N_CLICK_TYPES; i++) {
-	    w = glade_xml_get_widget (dd->xml, img_widgets[i]);
-
+	    w = WID (img_widgets[i]);
 	    if (alloc->height < 32) {
 		tmp = gdk_pixbuf_scale_simple (dd->click[i],
 					       alloc->height - 7,
 					       alloc->height - 7,
 					       GDK_INTERP_HYPER);
 		if (tmp) {
-		    gtk_image_set_from_pixbuf (GTK_IMAGE(w), tmp);
+		    gtk_image_set_from_pixbuf (GTK_IMAGE (w), tmp);
 		    g_object_unref (tmp);
 		}
 	    }
 	    else
-		gtk_image_set_from_pixbuf (GTK_IMAGE(w), dd->click[i]);
+		gtk_image_set_from_pixbuf (GTK_IMAGE (w), dd->click[i]);
 	}
-
+    }
     dd->button_width = alloc->width;
     dd->button_height = alloc->height;
 }
@@ -156,45 +272,55 @@ box_size_allocate (GtkWidget *widget, GtkAllocation *alloc, gpointer data)
 static void
 applet_orient_changed (PanelApplet *applet, guint orient, gpointer data)
 {
-    DwellData *dd = (DwellData *) data;
+    DwellData *dd = data;
+    gboolean dwell;
 
-    gtk_container_remove (GTK_CONTAINER (applet), dd->box);
+    gtk_container_remove (GTK_CONTAINER (applet), g_object_ref (dd->box));
 
     switch (orient) {
     case PANEL_APPLET_ORIENT_UP:
     case PANEL_APPLET_ORIENT_DOWN:
-	dd->box = glade_xml_get_widget (dd->xml, "box_hori");
-	dd->button = glade_xml_get_widget (dd->xml, "single_click");
+	dd->box = WID ("box_hori");
+	dd->ct_box = WID ("ct_box");
+	dd->enable = WID ("enable");
+	dd->button = WID ("single_click");
 	break;
     case PANEL_APPLET_ORIENT_LEFT:
     case PANEL_APPLET_ORIENT_RIGHT:
-	dd->box = glade_xml_get_widget (dd->xml, "box_vert");
-	dd->button = glade_xml_get_widget (dd->xml, "single_click_v");
+	dd->box = WID ("box_vert");
+	dd->ct_box = WID ("ct_box_v");
+	dd->enable = WID ("enable_v");
+	dd->button = WID ("single_click_v");
     default:
 	break;
     }
 
     if (dd->box->parent)
 	gtk_widget_reparent (dd->box, GTK_WIDGET (applet));
-    else
+    else {
 	gtk_container_add (GTK_CONTAINER (applet), dd->box);
+	g_object_unref (dd->box);
+    }
+
+    dwell = gconf_client_get_bool (dd->client, OPT_DWELL, NULL);
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dd->enable), dwell);
+    update_sensitivity (dd);
 }
 
 static void
 applet_unrealized (GtkWidget *widget, gpointer data)
 {
-    DwellData *dd = (DwellData *) data;
+    DwellData *dd = data;
     gint i;
 
     for (i = 0; i < N_CLICK_TYPES; i++)
-	g_object_unref (dd->click[i]);
+	if (dd->click[i])
+	    g_object_unref (dd->click[i]);
 
-    g_object_unref (glade_xml_get_widget (dd->xml, "box_vert"));
-    g_object_unref (glade_xml_get_widget (dd->xml, "box_hori"));
     g_object_unref (dd->client);
     g_object_unref (dd->proxy);
     g_object_unref (dd->xml);
-
+    g_timer_destroy (dd->timer);
     g_slice_free (DwellData, dd);
 }
 
@@ -203,12 +329,21 @@ preferences_dialog (BonoboUIComponent *component,
 		    gpointer           data,
 		    const char        *cname)
 {
-    g_spawn_command_line_async ("gnome-mouse-properties -p accessibility",
-				NULL);
+    GError *error = NULL;
+
+    if (!g_spawn_command_line_async ("gnome-mouse-properties -p accessibility",
+				     &error)) {
+	mt_common_show_dialog (_("Failed to Launch Mouse Preferences"),
+			       error->message,
+			       MT_MESSAGE_WARNING);
+	g_error_free (error);
+    }
 }
 
 static void
-help_dialog (BonoboUIComponent *component, gpointer data, const char *cname)
+help_dialog (BonoboUIComponent *component,
+	     gpointer           data,
+	     const char        *cname)
 {
     GError *error = NULL;
 
@@ -217,33 +352,29 @@ help_dialog (BonoboUIComponent *component, gpointer data, const char *cname)
 				     "mousetweaks",
 				     NULL,
 				     &error)) {
-	mt_show_dialog (_("Couldn't display help"),
-			error->message,
-			GTK_MESSAGE_ERROR);
+	mt_common_show_dialog (_("Failed to Display Help"),
+			       error->message,
+			       MT_MESSAGE_WARNING);
 	g_error_free (error);
     }
 }
 
 static void
-about_dialog (BonoboUIComponent *component, gpointer data, const char *cname)
+about_dialog (BonoboUIComponent *component,
+	      gpointer           data,
+	      const char        *cname)
 {
-    DwellData *dd = (DwellData *) data;
-    GtkWidget *about;
+    DwellData *dd = data;
 
-    about = glade_xml_get_widget (dd->xml, "about");
-
-    if (GTK_WIDGET_VISIBLE (about))
-	gtk_window_present (GTK_WINDOW (about));
-    else
-	gtk_widget_show (about);
+    gtk_window_present (GTK_WINDOW (WID ("about")));
 }
 
 static void
 about_response (GtkWidget *about, gint response, gpointer data)
 {
-    DwellData *dd = (DwellData *) data;
+    DwellData *dd = data;
 
-    gtk_widget_hide (glade_xml_get_widget (dd->xml, "about"));
+    gtk_widget_hide (WID ("about"));
 }
 
 static inline void
@@ -257,85 +388,82 @@ init_button (DwellData *dd, GtkWidget *button)
 }
 
 static void
-init_horizontal_box (DwellData *dd)
+setup_box (DwellData *dd)
 {
-    GtkWidget *w;
+    GtkWidget *widget;
+    GtkSizeGroup *hgroup, *vgroup;
     gint i;
 
-    w = glade_xml_get_widget (dd->xml, "box_hori");
-    g_object_ref (w);
+    /* horizontal */
+    hgroup = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+    widget = WID ("single_click");
+    init_button (dd, widget);
+    gtk_size_group_add_widget (hgroup, widget);
+    g_signal_connect (widget, "size-allocate",
+		      G_CALLBACK (button_size_allocate), dd);
 
-    w = glade_xml_get_widget (dd->xml, "single_click");
-    g_signal_connect (w, "size-allocate",
-		      G_CALLBACK (box_size_allocate), dd);
-    init_button (dd, w);
+    init_button (dd, WID ("double_click"));
+    init_button (dd, WID ("drag_click"));
+    init_button (dd, WID ("right_click"));
 
-    w = glade_xml_get_widget (dd->xml, "double_click");
-    init_button (dd, w);
+    widget = WID ("enable");
+    gtk_size_group_add_widget (hgroup, widget);
+    g_signal_connect (widget, "button-press-event",
+		      G_CALLBACK (do_not_eat), NULL);
+    g_signal_connect (widget, "toggled",
+		      G_CALLBACK (enable_dwell_changed), dd);
+    g_signal_connect (widget, "enter-notify-event",
+		      G_CALLBACK (enable_dwell_crossing), dd);
+    g_signal_connect (widget, "leave-notify-event",
+		      G_CALLBACK (enable_dwell_crossing), dd);
+    g_signal_connect_after (widget, "expose-event",
+			    G_CALLBACK (enable_dwell_exposed), dd);
 
-    w = glade_xml_get_widget (dd->xml, "drag_click");
-    init_button (dd, w);
+    /* vertical */
+    vgroup = gtk_size_group_new (GTK_SIZE_GROUP_VERTICAL);
+    widget = WID ("single_click_v");
+    gtk_size_group_add_widget (vgroup, widget);
+    init_button (dd, widget);
+    g_signal_connect (WID ("single_click_v"), "size-allocate",
+		      G_CALLBACK (button_size_allocate), dd);
+    init_button (dd, WID ("double_click_v"));
+    init_button (dd, WID ("drag_click_v"));
+    init_button (dd, WID ("right_click_v"));
 
-    w = glade_xml_get_widget (dd->xml, "right_click");
-    init_button (dd, w);
+    widget = WID ("enable_v");
+    gtk_size_group_add_widget (vgroup, widget);
+    g_signal_connect (widget, "button-press-event",
+		      G_CALLBACK (do_not_eat), NULL);
+    g_signal_connect (widget, "toggled",
+		      G_CALLBACK (enable_dwell_changed), dd);
+    g_signal_connect (widget, "enter-notify-event",
+		      G_CALLBACK (enable_dwell_crossing), dd);
+    g_signal_connect (widget, "leave-notify-event",
+		      G_CALLBACK (enable_dwell_crossing), dd);
+    g_signal_connect_after (widget, "expose-event",
+			    G_CALLBACK (enable_dwell_exposed), dd);
 
     for (i = 0; i < N_CLICK_TYPES; i++) {
-	w = glade_xml_get_widget (dd->xml, img_widgets[i]);
-	gtk_image_set_from_pixbuf (GTK_IMAGE(w), dd->click[i]);
+	gtk_image_set_from_pixbuf (GTK_IMAGE (WID (img_widgets[i])),
+				   dd->click[i]);
+	gtk_image_set_from_pixbuf (GTK_IMAGE (WID (img_widgets_v[i])),
+				   dd->click[i]);
     }
 }
 
-static void
-init_vertical_box (DwellData *dd)
-{
-    GtkWidget *w;
-    gint i;
-
-    w = glade_xml_get_widget (dd->xml, "box_vert");
-    g_object_ref (w);
-
-    w = glade_xml_get_widget (dd->xml, "single_click_v");
-    g_signal_connect (G_OBJECT(w), "size-allocate",
-		      G_CALLBACK(box_size_allocate), dd);
-    init_button (dd, w);
-
-    w = glade_xml_get_widget (dd->xml, "double_click_v");
-    init_button (dd, w);
-
-    w = glade_xml_get_widget (dd->xml, "drag_click_v");
-    init_button (dd, w);
-
-    w = glade_xml_get_widget (dd->xml, "right_click_v");
-    init_button (dd, w);
-
-    for (i = 0; i < N_CLICK_TYPES; i++) {
-	w = glade_xml_get_widget (dd->xml, img_widgets_v[i]);
-	gtk_image_set_from_pixbuf (GTK_IMAGE(w), dd->click[i]);
-    }
-}
-
-static void
-update_sensitivity (DwellData *dd)
-{
-    gint mode;
-    gboolean enabled, sensitive;
-
-    enabled = gconf_client_get_bool (dd->client, OPT_DWELL, NULL);
-    mode = gconf_client_get_int (dd->client, OPT_MODE, NULL);
-
-    sensitive = dd->active && enabled && mode == DWELL_MODE_CTW;
-    gtk_widget_set_sensitive (dd->box, sensitive);
-}
-
 static void
 clicktype_changed (DBusGProxy *proxy,
 		   guint       clicktype,
 		   gpointer    data)
 {
-    DwellData *dd = (DwellData *) data;
+    DwellData *dd = data;
     GtkToggleButton *button;
     GSList *group;
 
+    if (clicktype >= N_CLICK_TYPES)
+	return;
+
+    dd->cct = clicktype;
     group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (dd->button));
     button = GTK_TOGGLE_BUTTON (g_slist_nth_data (group, clicktype));
 
@@ -349,7 +477,7 @@ status_changed (DBusGProxy *proxy,
 		gboolean    status,
 		gpointer    data)
 {
-    DwellData *dd = (DwellData *) data;
+    DwellData *dd = data;
 
     dd->active = status;
     update_sensitivity (dd);
@@ -386,14 +514,45 @@ setup_dbus_proxy (DwellData *dd)
     return TRUE;
 }
 
+static gboolean
+mousetweaks_is_active (void)
+{
+    DBusGConnection *bus;
+    DBusGProxy *proxy;
+    gboolean result = FALSE;
+
+    bus = dbus_g_bus_get (DBUS_BUS_SESSION, NULL);
+    if (bus) {
+	proxy = dbus_g_proxy_new_for_name (bus,
+					   DBUS_SERVICE_DBUS,
+					   DBUS_PATH_DBUS,
+					   DBUS_INTERFACE_DBUS);
+	dbus_g_proxy_call (proxy, "NameHasOwner", NULL,
+			   G_TYPE_STRING, "org.gnome.Mousetweaks",
+			   G_TYPE_INVALID,
+			   G_TYPE_BOOLEAN, &result,
+			   G_TYPE_INVALID);
+	g_object_unref (proxy);
+    }
+    return result;
+}
+
 static void
 gconf_value_changed (GConfClient *client,
 		     const gchar *key,
-		     GConfValue *value,
-		     gpointer data)
+		     GConfValue  *value,
+		     gpointer     data)
 {
-    if (g_str_equal (key, OPT_MODE) || g_str_equal (key, OPT_DWELL))
-	update_sensitivity ((DwellData *) data);
+    DwellData *dd = data;
+
+    if (g_str_equal (key, OPT_MODE)) {
+	update_sensitivity (dd);
+    }
+    else if (g_str_equal (key, OPT_DWELL) && value->type == GCONF_VALUE_BOOL) {
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dd->enable),
+				      gconf_value_get_bool (value));
+	update_sensitivity (dd);
+    }
 }
 
 static gboolean
@@ -402,13 +561,12 @@ fill_applet (PanelApplet *applet)
     DwellData *dd;
     GtkWidget *about;
     PanelAppletOrient orient;
+    gboolean dwell;
 
     dd = g_slice_new0 (DwellData);
     if (!dd)
 	return FALSE;
 
-    dd->cct = DWELL_CLICK_TYPE_SINGLE;
-
     bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
     bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
     textdomain (GETTEXT_PACKAGE);
@@ -422,25 +580,33 @@ fill_applet (PanelApplet *applet)
 	return FALSE;
     }
 
+    /* dbus */
     if (!setup_dbus_proxy (dd)) {
 	g_object_unref (dd->xml);
 	g_slice_free (DwellData, dd);
 	return FALSE;
     }
 
-    about = glade_xml_get_widget (dd->xml, "about");
+    dd->active = mousetweaks_is_active ();
+    dd->cct = DWELL_CLICK_TYPE_SINGLE;
+    dd->timer = g_timer_new ();
+
+    /* about dialog */
+    about = WID ("about");
     g_object_set (about, "version", VERSION, NULL);
     g_signal_connect (about, "delete-event",
 		      G_CALLBACK (gtk_widget_hide_on_delete), NULL);
     g_signal_connect (about, "response",
 		      G_CALLBACK (about_response), dd);
 
+    /* gconf */
     dd->client = gconf_client_get_default ();
     gconf_client_add_dir (dd->client, MT_GCONF_HOME,
 			  GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
     g_signal_connect (dd->client, "value_changed",
 		      G_CALLBACK (gconf_value_changed), dd);
 
+    /* icons */
     dd->click[DWELL_CLICK_TYPE_SINGLE] =
 	gdk_pixbuf_new_from_file (DATADIR "/single-click.png", NULL);
     dd->click[DWELL_CLICK_TYPE_DOUBLE] =
@@ -450,6 +616,7 @@ fill_applet (PanelApplet *applet)
     dd->click[DWELL_CLICK_TYPE_RIGHT] =
 	gdk_pixbuf_new_from_file (DATADIR "/right-click.png", NULL);
 
+    /* applet initialization */
     panel_applet_set_flags (applet,
 			    PANEL_APPLET_EXPAND_MINOR |
 			    PANEL_APPLET_HAS_HANDLE);
@@ -463,23 +630,29 @@ fill_applet (PanelApplet *applet)
     g_signal_connect (applet, "unrealize",
 		      G_CALLBACK (applet_unrealized), dd);
 
+    /* check initial orientation */
     orient = panel_applet_get_orient (applet);
     if (orient == PANEL_APPLET_ORIENT_UP ||
 	orient == PANEL_APPLET_ORIENT_DOWN) {
-	dd->box = glade_xml_get_widget (dd->xml, "box_hori");
-	dd->button = glade_xml_get_widget (dd->xml, "single_click");
+	dd->box = WID ("box_hori");
+	dd->ct_box = WID ("ct_box");
+	dd->enable = WID ("enable");
+	dd->button = WID ("single_click");
     }
     else {
-	dd->box = glade_xml_get_widget (dd->xml, "box_vert");
-	dd->button = glade_xml_get_widget (dd->xml, "single_click_v");
+	dd->box = WID ("box_vert");
+	dd->ct_box = WID ("ct_box_v");
+	dd->enable = WID ("enable_v");
+	dd->button = WID ("single_click_v");
     }
 
-    init_horizontal_box (dd);
-    init_vertical_box (dd);
+    dwell = gconf_client_get_bool (dd->client, OPT_DWELL, NULL);
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dd->enable), dwell);
+    gtk_button_set_label (GTK_BUTTON (dd->enable), dwell ? _("On") : _("Off"));
 
+    setup_box (dd);
     gtk_widget_reparent (dd->box, GTK_WIDGET (applet));
     gtk_widget_show (GTK_WIDGET (applet));
-
     update_sensitivity (dd);
 
     return TRUE;
